#!/bin/sh

#  lbd.sh (Little Build Daemon)
#  This shell script rebuilds distributions of (no)src.rpm files
#
#  Copyright (c) by Stefan van der Eijk <stefan@eijk.nu>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#  2005/08/07 Stefan van der Eijk <stefan@eijk.nu>
#  - add check_config
#  - don't use update_chroot as it messes up the order in which the chroots
#    are updated, same applies to REGENERATE
#  - add switch to not build src.rpm's that produce noarch.rpm packages.
#    To build these packages add "BUILD_NOARCH=1" to configuration of 
#    that source.
#  - add date to filename of logfile
#  - merge urpmi and urpmi_error output files into buildoutput file
#  - mount /dev/pts and /proc from the native system to the chroot
#    before building, unmount them after building
#  - replaced __KEEP_BUILDDIR construction with KEEP_BUILDDIR=2
#  - instroduced BUILD_RESULT to be able to determin where the buildoutput will
#    be put, then remove the buildrequires, then actually move the buildoutput
#  - remove min() --> it wasn't used anymore
#  - renamed urpmi_media() to update_media()
#  - renmaed "urpmi" variable to "media" to be a bit smart friendlier
#  - revert from $OLD_TARGET --> not documented
#  - some other misc changes
#
#  2005/05/12 Christiaan Welvaart <cjw@daneel.dyndns.org>
#  - new function update_chroot
#    separate updating of media and upgrading of chroots
#  - new options:
#    * FORCE_KEEP_BUILDDIR
#      save build dir(s) even if build succeeds
#    * REGENERATE (for medium)
#       medium is an rpm dir; rebuild hdlist when necessary
#    * UPDATE (for target)
#      update the chroot for this target (using the target's media list)
#    * BUILD_OLD
#      set to "no" to only build new packages
#
#	2005/05/11 Stefan van der Eijk <stefan@eijk.nu>
#	- also move noarch.rpm packages
#	- disable min() routine
#
#	2005/03/28 Stefan van der Eijk <stefan@eijk.nu>
#	- fix smart support
#
#	2005/03/10 Stefan van der Eijk <stefan@eijk.nu>
#	- add srpms_mismatch check
#	- sleep 5
#
#	2005/03/07 Stefan van der Eijk <stefan@eijk.nu>
#	- changed ${SLBD} to ${LBD}
#	- sort_on_buildtime (Christiaan Welvaart)
#	- misc apt changes (Christiaan Welvaart)
#
#	2005/03/01 Stefan van der Eijk <stefan@eijk.nu>
#	- renamed config file from ~/.slbd to ~/.lbd
#
#	2005/03/01 Stefan van der Eijk <stefan@eijk.nu>
#	- use same date format
#	- detect dependency loop
#	- add multiarch check
#	- add smart support
#
#	2004/10/01 Stefan van der Eijk <stefan@eijk.nu>
#	- Also switch .rpmmacros file for lbd user on the native system.
#	  It was already being switched in the chroot. This is needed for
#	  building the src.rpm from where the BuildRequires will be installed.
#
# v0.27 2004/08/03
#	- check to see if other hosts aren't searching for new packages to build
#
# v0.26 2004/07/29
#	- add support for nosrc.rpm packages
#
# v0.25 2004/07/21
#	- add KEEP_BUILDDIR variable, to save the contents of the builddir
#	- add rpmlint checks
#
# v0.24 2004/07/11
#	- Misc fedora changes
#
# v0.23 2004/06/18
#	- added filter for "Internal compiler error"
#
# v0.22 2004/05/21
#	- put segfault buildoutputs in a seperate dir
#	- remove CHROOT=0 and CHROOT variable, chroots are always used
#	- introduce INSTALLER variable
#
# v0.21 2004/04/30
#	- introduces RPM_OPTIONS variable, to cope with fedora's rpmbuild
#
# v0.20 2004/04/18
#	- make paths to executables variable
#	- changed the name of the original ${RPM_BIN} variable to ${COMPARE}
#	- make path to slbd base variable ${SLBD}
#
# v0.19 2004/04/12
#	- sort out packages with dependency issues
#	- add missing dependency to buildoutput
#
# v.018 2004/01/05
#	- invoke urpmi with "--keep" option when updating a built environment
#
# v.017 2004/01/02
#	- only check for Segmentation fault on problem packages
#
# v0.16 2003/12/23
#	- fixed hardcoded /chroot/cooker --> ${CHROOT_PATH} (vdanen)
#	- use unified diff for Filelist, Provides and Requires comparison
#
# v0.15 2003/12/15
#	- put the Segfault check before the other BuildOutput checks
#
# v0.14 2003/11/26
#	- changed the direcotry structure
#
# v0.13 2003/11/22
#	- added rpm_qa directories to contain package listing at build time
#
# v0.12 2003/11/18
#	- added filter for "Segmentaion fault"
#
# v0.11 2003/11/13
#	- rebuild src.rpm to obtain correct dependency information
#
# v0.10 2003/10/31
#	- add KEEP_RPMS
#
# v0.09 2003/10/18
#	- invoke urpmi with --no-uninstall
#
# v0.08 2003/09/07
#	- make automake1.7 hack simpler
#	- added "File not found" check
#
# v0.07 2003/09/02
#	- quite some small fixes
#	- automake1.7 hack
#
# v0.06 2003/07/06
#	- added support for:
#	  - configuration file
#	  - multiple instances on one machine (different chroots)
#	- merged min.sh
#
# v0.05 2003/05/26
#	- implemented chroot support
#	- rewrite of find_new() and find_old()
#
# v0.04 2003/05/02
#	- different urpmi locations per arch
#	- delete only ${HOSTTYPE} packages, not noarch
#
# v0.03 2003/04/27
#	- add number of packages installed to the log
#	- log when min.sh is run
#
# v0.03 2003/04/22
#	- log what packages rebuilt
#	- try to recover initial state by running min.sh
#
# v0.02 2003/04/22
#	- check if number of packages is the same before & after build
#	otherwise remove /build/run/`hostname -f` file
#	- invoke urpmi with -q option
#
# v0.01 2003/04/06
#	- first release under the GPL
#

export PATH=$PATH:/usr/X11R6/bin:/usr/sbin:/sbin

check_config () {
echo "Checking sources"
for a in `~/.lbd src`
do
	. ~/.lbd ${a}
	if [ "${BUILD_NOARCH}" = "" ]
	then
		echo "${a} Optional parameter \"BUILD_NOARCH\" not set"
	elif [ "${BUILD_NOARCH}" = "1" ]
	then
		echo "${a} Optional parameter \"BUILD_NOARCH\" set to \"1\""
	else
		echo "${a} Optional parameter \"BUILD_NOARCH\" set to \"${BUILD_NOARCH}\""
	fi
	if [ "${BUILD_OUTPUT}" = "" ]
	then
		echo "${a} Required parameter \"BUILD_OUTPUT\" not set - ERROR"
	elif [ -d ${BUILD_OUTPUT} ]
	then
		echo "${a} Required parameter \"BUILD_OUTPUT\": directory ${BUILD_OUTPUT} exists"
	else
		echo "${a} Required parameter \"BUILD_OUTPUT\": directory ${BUILD_OUTPUT} doesn't exist - ERROR"
	fi
	if [ "${CHROOT_PATH}" = "" ]
	then
		echo "${a} Required parameter \"CHROOT_PATH\" not set - ERROR"
	elif [ -d ${CHROOT_PATH} ]
	then
		echo "${a} Required parameter \"CHROOT_PATH\": directory ${CHROOT_PATH} exists"
	else
		echo "${a} Required parameter \"CHROOT_PATH\": directory ${CHROOT_PATH} doesn't exist - ERROR"
	fi
	if [ "${COMPARE}" = "" ]
	then
		echo "${a} Optional parameter \"COMPARE\" not set"
	elif [ -d ${COMPARE} ]
	then
		echo "${a} Optional parameter \"COMPARE\": directory ${COMPARE} exists"
	else
		echo "${a} Optional parameter \"COMPARE\": directory ${COMPARE} doesn't exist"
	fi
	if [ "${INSTALLER}" = "apt" ] \
		|| [ "${INSTALLER}" = "smart" ] \
		|| [ "${INSTALLER}" = "urpmi" ]
	then
		echo "${a} Required parameter \"INSTALLER\" set to ${INSTALLER}"
	else
		echo "${a} Required parameter \"INSTALLER\" not set, or set to an unkown value - ERROR"
	fi
	if [ "${KEEP_BUILDDIR}" = "" ]
	then
		echo "${a} Optional parameter \"KEEP_BUILDDIR\" not set"
	elif [ "${KEEP_BUILDDIR}" = "1" ]
	then
		echo "${a} Optional parameter \"KEEP_BUILDDIR\" set to \"1\""
	elif [ "${KEEP_BUILDDIR}" = "2" ]
	then
		echo "${a} Optional parameter \"KEEP_BUILDDIR\" set to \"2\""
	else
		echo "${a} Optional parameter \"KEEP_BUILDDIR\" set to an unkown value - ERROR"
	fi
	if [ "${KEEP_RPMS}" = "" ]
	then
		echo "${a} Optional parameter \"KEEP_RPMS\" not set"
	elif [ "${KEEP_RPMS}" = "1" ]
	then
		echo "${a} Optional parameter \"KEEP_RPMS\" set to \"1\""
	else
		echo "${a} Optional parameter \"KEEP_RPMS\" set to an unkown value - ERROR"
	fi
# Todo for ${MEDIA}, check if it exists in apt / smart / urpmi 
	if [ "${MEDIA}" = "" ]
	then
		echo "${a} Required parameter \"MEDIA\" not set - ERROR"
	else
		for b in `echo ${MEDIA} | perl -p -e "s#,#\n#g" 2> /dev/null`
		do
			. ~/.lbd media_${b}
			if [ "${MEDIA_PATH}" = "" ]
			then
				echo "${b} Required parameter \"MEDIA_PATH\" for MEDIA ${b} not set - ERROR"
			elif [ -e ${MEDIA_PATH} ]
			then
				echo "${b} Required parameter \"MEDIA_PATH\" for MEDIA ${b}: ${MEDIA_PATH} exists"
			else
				echo "${b} Required parameter \"MEDIA_PATH\" for MEDIA ${b}: ${MEDIA_PATH} doesn't exist - ERROR"
			fi
		done
	fi
	if [ "${MTIME}" = "" ]
	then
		echo "${a} Required parameter \"MTIME\" not set - ERROR"
	else
		echo "${a} Required parameter \"MTIME\" set to ${MTIME}"
	fi
	if [ "${REDUNDANT}" = "" ]
	then
		echo "${a} Optional parameter \"REDUNDANT\" not set"
	elif [ "${REDUNDANT}" = "1" ]
	then
		echo "${a} Optional parameter \"REDUNDANT\" set to \"1\""
	else
		echo "${a} Optional parameter \"REDUNDANT\" or set to an unkown value - ERROR"
	fi
	if [ "${RPM_MACROS}" = "" ]
	then
		echo "${a} Required parameter \"RPM_MACROS\" not set - ERROR"
	elif [ -f ~/${RPM_MACROS} ]
	then
		echo "${a} Required parameter \"RPM_MACROS\": file ~/${RPM_MACROS} exists"
	else
		echo "${a} Required parameter \"RPM_MACROS\": file ~/${RPM_MACROS} doesn't exist - ERROR"
	fi
	if [ "${RPM_OPTIONS}" = "" ]
	then
		echo "${a} Optional parameter \"RPM_OPTIONS\" not set"
	else
		echo "${a} Optional parameter \"RPM_OPTIONS\" set to ${RPM_OPTIONS}"
	fi
	if [ "${RPM_SRC}" = "" ]
	then
		echo "${a} Required parameter \"RPM_SRC\" not set - ERROR"
	elif [ -d ${RPM_SRC} ]
	then
		echo "${a} Required parameter \"RPM_SRC\": directory ${RPM_SRC} exists"
	else
		echo "${a} Required parameter \"RPM_SRC\": directory ${RPM_SRC} doesn't exist - ERROR"
	fi
	if [ "${STRACE}" = "" ]
	then
		echo "${a} Optional parameter \"STRACE\" not set"
	elif [ "${STRACE}" = "1" ]
	then
		echo "${a} Optional parameter \"STRACE\" set to \"1\""
	else
		echo "${a} Optional parameter \"STRACE\" set to an unkown value - ERROR"
	fi
done
}

cleanup () {
# Cleanup build environment
# Input: src name
# - ${1} = src name
# Output: none
# Configuration file: ~/.lbd
# - CHROOT_PATH

. ~/.lbd ${src}

echo "*** Cleanup ***"

NHOME=${CHROOT_PATH}${HOME}
rm -rf ${CHROOT_PATH}/tmp/* ${CHROOT_PATH}/var/tmp/*
rm -rf	${HOME}/RPM/BUILD/* \
	${HOME}/RPM/RPMS/{${P_ARCH},noarch}/* \
	${HOME}/RPM/SOURCES/* \
	${HOME}/RPM/SPECS/* \
	${HOME}/tmp/*
find	${HOME}/RPM/ -type f -exec rm -rf {} \;

chmod 777 ${NHOME}/RPM/BUILD/* -R 
rm -rf	${NHOME}/RPM/BUILD/* \
	${NHOME}/RPM/RPMS/{${P_ARCH},noarch}/* \
	${NHOME}/RPM/SOURCES/* \
	${NHOME}/RPM/SPECS/* \
	${NHOME}/tmp/*
find	${NHOME}/RPM/ -type f -exec rm -rf {} \;
}

find_new () {
# Find a new (no)src.rpm to build.
# Input: src name
# - ${1} = src name
# Output: name of (no)src.rpm
# Configuration file: ~/.lbd
# - BUILD_OUTPUT
# - LBD
# - P_ARCH
# - RPM_SRC
# - SORT_ON_BUILDTIME (optional)

src=${1}

. ~/.lbd ${src}

while [ -f ${LBD}/${P_ARCH}/searching ]
do
	sleep 5
done
echo `hostname -f` > ${LBD}/${P_ARCH}/searching

find ${BUILD_OUTPUT}/BO/ -type f \
| perl -p -e "s#.*/##g" 2> /dev/null \
| sort -u \
> ~/tmp/${src}_LOG1

find ${BUILD_OUTPUT}/ -type f \
| grep -v RPMS \
| perl -p -e "s#.*/##g" 2> /dev/null \
| sort -u \
> ~/tmp/${src}_LOG2

find ${RPM_SRC}/ -type f \
| grep src.rpm$ \
| perl -p -e "s#.*/##g" 2> /dev/null \
| perl -p -e "s#^\..*\n##g" 2> /dev/null \
| perl -p -e "s#\.(nosrc|src)\.rpm##g" 2> /dev/null \
| sort -u \
> ~/tmp/${src}_LOG3

diff ~/tmp/${src}_LOG1 ~/tmp/${src}_LOG2 \
> ~/tmp/${src}_diff

diff ~/tmp/${src}_LOG1 ~/tmp/${src}_LOG3 \
>> ~/tmp/${src}_diff

diff ~/tmp/${src}_LOG2 ~/tmp/${src}_LOG3 \
>> ~/tmp/${src}_diff

if [ "${SORT_ON_BUILDTIME}" = "1" ] && [ -x `dirname $0`/sort_on_buildtime ]
then
# TODO: old logs are not removed in this mode
	touch ${BUILD_OUTPUT}/SRPMS.queue
	QUEUESIZE=`wc -l ${BUILD_OUTPUT}/SRPMS.queue | cut -d\  -f1`
	if [ ! \( ${QUEUESIZE} -gt 20 \) ]
	then
		cat ~/tmp/${src}_diff \
		| grep "[>|<]" \
		| perl -p -e "s#\\S\\s+(\\S+)#\\1#g" 2> /dev/null \
		| `dirname $0`/sort_on_buildtime $RPM_SRC 2>/dev/null \
		| perl -p -e "s#.*/##g" 2> /dev/null \
		| perl -p -e "s#^\..*\n##g" 2> /dev/null \
		| perl -p -e "s#\.src\.rpm##g" 2> /dev/null \
		> ${BUILD_OUTPUT}/SRPMS.queue

		QUEUESIZE=`wc -l ${BUILD_OUTPUT}/SRPMS.queue | cut -d\  -f1`
	fi

	RESULT=`head -n 1 ${BUILD_OUTPUT}/SRPMS.queue`
	tail -n `expr $QUEUESIZE - 1` ${BUILD_OUTPUT}/SRPMS.queue >${BUILD_OUTPUT}/SRPMS.queue.new
	mv -f ${BUILD_OUTPUT}/SRPMS.queue.new ${BUILD_OUTPUT}/SRPMS.queue
else
	RESULT=`cat ~/tmp/${src}_diff \
	| grep "[>|<]" \
	| perl -p -e "s#\S\s+(\S+)#\1#g" 2> /dev/null \
	| sort -u \
	| head -n 1`
fi

if [ "${RESULT}" = "" ]
then
	rm -rf \
	~/tmp/${src}_LOG? \
	~/tmp/${src}_diff \
	${LBD}/${P_ARCH}/searching \
	2> /dev/null
else
	rm -rf \
	~/tmp/${src}_LOG? \
	~/tmp/${src}_diff \
	2> /dev/null
	echo ${RESULT}
fi
}

find_old () {
# Find an old (no)src.rpm to build.
# Input: src name
# - ${1} = src name
# Output: name of (no)src.rpm
# Configuration file: ~/.lbd
# - BUILD_OUTPUT
# - LBD
# - MTIME
# - P_ARCH

src=${1}

. ~/.lbd ${src}

while [ -f ${LBD}/${P_ARCH}/searching ]
do
	sleep 5
done
echo `hostname -f` > ${LBD}/${P_ARCH}/searching

find \
${BUILD_OUTPUT}/BO/ \
${BUILD_OUTPUT}/BO/dependency/ \
-maxdepth 1 \
-mtime +1 \
-type f \
| perl -p -e "s#.*/(\S+)#\1#g" 2> /dev/null \
>> ~/tmp/${src}_BUILD

find \
${BUILD_OUTPUT}/BO/cannot_be_installed/ \
${BUILD_OUTPUT}/BO/debug/ \
${BUILD_OUTPUT}/BO/file_not_found/ \
${BUILD_OUTPUT}/BO/noarch/ \
${BUILD_OUTPUT}/BO/OK/ \
${BUILD_OUTPUT}/BO/problem/ \
${BUILD_OUTPUT}/BO/unpackaged_files/ \
-mtime ${MTIME} \
-type f \
| perl -p -e "s#.*/(\S+)#\1#g" 2> /dev/null \
>> ~/tmp/${src}_BUILD

RESULT=`sort -u ~/tmp/${src}_BUILD | uniq | head -n 1`

if [ "${RESULT}" = "" ]
then
	rm -rf \
	~/tmp/${src}_BUILD \
	${LBD}/${P_ARCH}/searching \
	2> /dev/null
else
	echo ${RESULT}
	rm -rf \
	~/tmp/${src}_BUILD \
	2> /dev/null
fi
}

build () {
# Build a package
# Input:
# - ${1} = package name
# - ${2} = src name
# Output: none
# Configuration file: ~/.lbd
# - BUILD_OUTPUT	- required
# - CHROOT_PATH		- required
# - COMPARE		- optional
# - INSTALLER		- required
# - KEEP_BUILDDIR	- optional
# - MEDIA		- required
# - RPM_SRC		- required
# - STRACE		- optional

pkg=${1}
src=${2}

. ~/.lbd ${src}

echo "*** Going for: i=${i} j=${j} P_ARCH=${P_ARCH} pkg=${pkg} src=${src} ***"

# The previously build rpms are removed from the ${BUILD_OUTPUT}/RPMS/ directory.
if	[ -e ${BUILD_OUTPUT}/BO/OK/${pkg} ]
then
	for i in `fgrep "Wrote:" ${BUILD_OUTPUT}/BO/OK/${pkg} | perl -p -e "s#.*/##g"`
	do
		if [ -e ${BUILD_OUTPUT}/RPMS/${i} ]
		then
			echo `date -R` `rm -frv ${BUILD_OUTPUT}/RPMS/${i}` \
			>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
		fi
	done
fi

# The old buildoutputs, etc are removed.
find ${BUILD_OUTPUT}/ -name ${pkg} \
| grep -v RPMS \
| xargs rm -rf

# Test if ${RPM_SRC}/${pkg}.(no)src.rpm exists, if the file doesn't exist, 
# no need to rebuild.
unset SRC_EXT
if	[ -e ${RPM_SRC}/${pkg}.nosrc.rpm ];
then
	SRC_EXT=nosrc
elif	[ -e ${RPM_SRC}/${pkg}.src.rpm ];
then
	SRC_EXT=src
fi

if	[ "${SRC_EXT}" = "nosrc" ] \
	|| [ "${SRC_EXT}" = "src" ];
then

# Check if src.rpm will product noarch packages
# if BUILD_NOARCH is not set to "1", touch the buildoutput file
# and don't rebuild the package

	if	[ `rpm -qp --qf %{BUILDARCHS} ${RPM_SRC}/${pkg}.${SRC_EXT}.rpm` = "noarch" ] \
		&& [ "${BUILD_NOARCH}" != "1" ]
	then
		touch ${BUILD_OUTPUT}/BO/noarch/${pkg}
		rm -rf ${LBD}/${P_ARCH}/searching
		continue 3 
	fi

# Flip & copy over the .rpmmacros files
	rm -f ${HOME}/.rpmmacros
	opwd=`pwd`
	cd ${HOME}
	ln -s ${RPM_MACROS} .rpmmacros
	cd ${opwd}

	NHOME=${CHROOT_PATH}${HOME}
	rm -f ${NHOME}/.rpmmacros
	cp ${HOME}/${RPM_MACROS} ${NHOME}/.rpmmacros

# Install BuildRequires

	hostname -f > ${BUILD_OUTPUT}/BO/${pkg}
	rm -rf ${LBD}/${P_ARCH}/searching

	rpm -qa --root ${CHROOT_PATH} | sort > ~/tmp/begin.txt
	begin=`cat ~/tmp/begin.txt | wc -l`

	echo `date -R` ${src} ${pkg} ${begin} \
		| perl -p -e "s#\.src\.rpm##g" \
		2> /dev/null \
		>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt

# Dependency information is stored in the headers of a rpm package, and are generated
# when the package is built. To get the correct BuildRequires for the (no)src.rpm to be
# rebuilt, a new (no)src.rpm needs to be built, on the correct platform and with the correct
# (build) options specified.
	echo "*** Building ${j} (no)src.rpm ***"
	SPEC=`rpm -qpl ${RPM_SRC}/${pkg}.${SRC_EXT}.rpm 2> /dev/null \
		| grep "\.spec$"`
	rpm -i ${RPM_SRC}/${pkg}.${SRC_EXT}.rpm 2> /dev/null

# In case of a nosrc.rpm package, copy the missing sourcefiles to the SOURCES directories
# of the lbd homedir on the native system and the chroot.
	if [ "${SRC_EXT}" = "nosrc" ];
	then
		unset EXIT
		FILE=`rpm -bs --nodeps ${HOME}/RPM/SPECS/${SPEC} 2>&1 \
		| grep "No such file or directory" \
		| perl -p -e "s#.*/##g" \
		| cut -d ":" -f1 \
		| head -n1`
		while [ "${FILE}" != "" ] \
		&& [ "${EXIT}" != "1" ]
		do
			if [ -f "${LBD}/NOSRC/${FILE}" ]
			then
				cp -v ${LBD}/NOSRC/${FILE} ${HOME}/RPM/SOURCES/
				cp -v ${LBD}/NOSRC/${FILE} ${CHROOT_PATH}${HOME}/RPM/SOURCES/
				FILE=`rpm -bs --nodeps ${HOME}/RPM/SPECS/${SPEC} 2>&1 \
				| grep "No such file or directory" \
				| perl -p -e "s#.*/##g" \
				| cut -d ":" -f1 \
				| head -n1`
			else
				echo "${pkg} missing file: ${FILE}" >> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
				EXIT=1
			fi
		done
	fi

	SRPMS=`${RPM_REBUILD_BIN} -bs --nodeps ${HOME}/RPM/SPECS/${SPEC} ${RPM_OPTIONS} 2> /dev/null | cut -d " " -f2`

	echo "*** Installing BuildRequires for:" ${pkg} "***"

	if	[ "${INSTALLER}" = "apt" ]
	then
		${SUDO_BIN} ${APT_BIN} \
			build-dep \
			${SRPMS} \
			--assume-yes \
			--no-remove \
			-o RPM::RootDir=${CHROOT_PATH} \
			>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1

	elif	[ "${INSTALLER}" = "smart" ]
	then
		for i in `rpm -qpR ${SRPMS} \
			| grep -v "rpmlib(CompressedFileNames)"`
		do
			${SUDO_BIN} ${SMART_BIN} \
				-o force-channels=rpm-sys,${MEDIA} \
				-o rpm-root=${CHROOT_PATH} \
				install \
				-y \
				${i} \
				>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1
		done

	elif	[ "${INSTALLER}" = "urpmi" ]
	then
		${SUDO_BIN} ${URPMI_BIN} -q -p \
			${SRPMS} \
			--auto \
			--media ${MEDIA} \
			--no-uninstall \
			--no-verify-rpm \
			--root ${CHROOT_PATH} \
			--split-length 0 \
			>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1

	elif	[ "${INSTALLER}" = "yum" ]
	then
		echo "*** not implemented yet ***"
		${SUDO_BIN} ${YUM_BIN} \
			-c ${YUM_CONFIG} \
			--installroot=${CHROOT_PATH}
	fi

# Optional: Find the Redundant BuildRequires
	if	[ "${REDUNDANT}" = 1 ] && \
		[ "${INSTALLER}" = "urpmi" ]
	then
		echo "*** Computing Redundant BuildRequires for:" ${pkg} "***"
		mkdir ~/tmp/${pkg}

		for a in basesystem rpm-build `rpm -qpR ${SRPMS} \
			| cut -d " " -f1 \
			| grep -v rpmlib \
			| sort -u`
		do
			for b in `urpmq -pr ${a} --media ${MEDIA} 2> /dev/null \
				| perl -p -e "s#\|#\n#g" 2> /dev/null \
				| grep -v "no package named ${a}" \
				| cut -d ":" -f2 \
				| sort -u`
			do
				for c in `urpmq -dpr ${b} --media ${MEDIA} 2> /dev/null \
					| perl -p -e "s#\|#\n#g" 2> /dev/null \
					| sort -u`
				do
					if [ "${b}" != "${c}" ]
					then
						echo ${c}
					fi
				done
			done | sort -u > ~/tmp/${pkg}/${a}
		done

# Find the obselete BuildRequires: 
		for a in `rpm -qpR ${SRPMS} \
			| cut -d " " -f1 \
			| grep -v rpmlib \
			| sort -u`
		do
			u=0
			for b in `urpmq -pr ${a} --media ${MEDIA} 2> /dev/null \
				| perl -p -e "s#\|#\n#g" 2> /dev/null \
				| grep -v "no package named ${a}" \
				| cut -d ":" -f2 \
				| sort -u`
			do
				for c in `cat ~/tmp/${pkg}/* | sort -u`
				do
					if [ "${b}" = "${c}" ]
					then
						u=1
					fi
				done
			done
			if [ "${u}" = 1 ]
			then
				echo "@@@ Redundant BuildRequires:" ${a} "@@@" \
				`fgrep ${b} ~/tmp/${pkg}/* \
				| cut -d ":" -f1 \
				| perl -p -e "s#.*/##g" 2> /dev/null \
				| perl -p -e "s#\n# #g" 2> /dev/null`
			fi
		done \
		> ${BUILD_OUTPUT}/redundant/${pkg} 2>&1
		rm -rf ~/tmp/${pkg}

		if [ `cat	${BUILD_OUTPUT}/redundant/${pkg}	| wc -l` = 0 ]
		then
			rm -rf	${BUILD_OUTPUT}/redundant/${pkg}
		fi
	fi

# Start building
	echo "*** Building:" ${pkg} "***"

	${SUDO_BIN} /bin/mount devpts ${CHROOT_PATH}/dev/pts -t devpts
	${SUDO_BIN} /bin/mount proc ${CHROOT_PATH}/proc/ -t proc

# Rebuild the (no)src.rpm, write output to file.
	if [ "${STRACE}" != "1" ]
	then
		rpm -qa --root ${CHROOT_PATH} | sort > ${BUILD_OUTPUT}/rpm_qa/${pkg}

		cp ${RPM_SRC}/${pkg}.${SRC_EXT}.rpm \
		${CHROOT_PATH}/var/tmp/

		${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
		su -l ${LOGNAME} \
		-c "${RPM_REBUILD_BIN} --rebuild \
	       	/var/tmp/${pkg}.${SRC_EXT}.rpm \
		${RPM_OPTIONS}" \
		>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1

		mv ${NHOME}/RPM/RPMS/${P_ARCH}/* \
		${HOME}/RPM/RPMS/${P_ARCH}/ 2> /dev/null
		mv ${NHOME}/RPM/RPMS/noarch/* \
		${HOME}/RPM/RPMS/noarch/ 2> /dev/null

		rm -rf ${CHROOT_PATH}/var/tmp/${pkg}.${SRC_EXT}.rpm
 	fi

	${SUDO_BIN} /bin/umount ${CHROOT_PATH}/dev/pts
	${SUDO_BIN} /bin/umount ${CHROOT_PATH}/proc
# Analyze the buildoutput, and move it to the right directory.

# Buildoutput end with "+ exit 0", build is successful.
	if [ "$(tail --lines 1 ${BUILD_OUTPUT}/BO/${pkg})" = "+ exit 0" ];
	then
# Find the "Wrote:" statements in the buildoutput.
		for n in `fgrep Wrote: ${BUILD_OUTPUT}/BO/${pkg} | cut -b 8- `
			do

			o=`echo ${n}	| perl -p -e "s#.*/##g" 2> /dev/null \
					| perl -p -e "s#\.(alpha|amd64|ia64|ppc|sparc|sparc64|x86_64)\.rpm#\.i586\.rpm#g" 2> /dev/null`

# Dependency loop
			for i in `rpm -qp --provides ${n} | cut -d " " -f1`
			do
				rpm -q ${i} \
				--root ${CHROOT_PATH} \
				--whatprovides \
				| grep -v "no package provides"
			done >> ${BUILD_OUTPUT}/deploop/${pkg}

			if [ -e ${COMPARE}/${o} ]
			then
# Filelist
				rpm -qpl ${COMPARE}/${o} \
					| sort -u > ~/tmp/${o}-filelist.old
				rpm -qpl ${n} \
					| sort -u > ~/tmp/${o}-filelist.new
				if [ `diff ~/tmp/${o}-filelist.old ~/tmp/${o}-filelist.new | wc -l` != 0 ]
				then
					echo "Filelist difference in" ${o} >> ${BUILD_OUTPUT}/filelist/${pkg}
					diff -uN ~/tmp/${o}-filelist.old ~/tmp/${o}-filelist.new \
						>> ${BUILD_OUTPUT}/filelist/${pkg}
				fi
				rm -rf ~/tmp/${o}-filelist.*

# Provides
				rpm -qp --provides ${COMPARE}/${o} \
					| sort -u > ~/tmp/${o}-provides.old
				rpm -qp --provides ${n} \
					| perl -p -e "s#\(64bit\)##g"				2> /dev/null \
					| perl -p -e "s#\(\)\##g"				2> /dev/null \
					| sort -u > ~/tmp/${o}-provides.new
				if [ `diff ~/tmp/${o}-provides.old ~/tmp/${o}-provides.new | wc -l` != 0 ]
				then
					echo "Provides" ${o} >> ${BUILD_OUTPUT}/provides/${pkg}
					diff -uN ~/tmp/${o}-provides.old ~/tmp/${o}-provides.new \
						>> ${BUILD_OUTPUT}/provides/${pkg}
				fi
				rm -rf ~/tmp/${o}-provides.*

# Requires
				rpm -qp --requires ${COMPARE}/${o} \
					| sort -u > ~/tmp/${o}-requires.old
				rpm -qp --requires ${n} \
					| perl -p -e "s#(64bit\)##g"				2> /dev/null \
					| perl -p -e "s#\(\)\##g"				2> /dev/null \
					| sort -u > ~/tmp/${o}-requires.new
#					| perl -p -e "s#libc\.so\.6\.1#libc\.so\.6#g"		2> /dev/null \
#					| perl -p -e "s#libcrypt\.so\.1\.1#libcrypt\.so\.1#g"	2> /dev/null \
#					| perl -p -e "s#libdl\.so\.2\.1#libdl\.so\.2#g"		2> /dev/null \
#					| perl -p -e "s#libm\.so\.6\.1#libm\.so\.6#g"		2> /dev/null \
#					| perl -p -e "s#libnsl\.so\.1\.1#libnsl\.so\.1#g"	2> /dev/null \
#					| perl -p -e "s#libresolv\.so\.2\.1#libresolv\.so\.2#g"	2> /dev/null \
#					| perl -p -e "s#libutil\.so\.1\.1#libutil\.so\.1#g"	2> /dev/null \
				if [ `diff ~/tmp/${o}-requires.old ~/tmp/${o}-requires.new | wc -l` != 0 ]
				then
					echo "Requires" ${o} >> ${BUILD_OUTPUT}/requires/${pkg}
					diff -uN ~/tmp/${o}-requires.old ~/tmp/${o}-requires.new \
						>> ${BUILD_OUTPUT}/requires/${pkg}
				fi
				rm -rf ~/tmp/${o}-requires.*

# Rpmlint
				rpmlint ${COMPARE}/${o} \
					| sort -u > ~/tmp/${o}-rpmlint.old
				rpmlint ${n} \
					| grep -v "no-packager-tag" \
					| grep -v "invalid-buildhost" \
					| grep -v "no-signature" \
					| sort -u > ~/tmp/${o}-rpmlint.new
				if [ `diff ~/tmp/${o}-rpmlint.old ~/tmp/${o}-rpmlint.new | wc -l` != 0 ]
				then
					echo "Rpmlint" ${o} >> ${BUILD_OUTPUT}/rpmlint/${pkg}
					diff -uN ~/tmp/${o}-rpmlint.old ~/tmp/${o}-rpmlint.new \
					>> ${BUILD_OUTPUT}/rpmlint_diff/${pkg}
				fi
				rm -rf ~/tmp/${o}-rpmlint.*
			fi
			rpmlint ${n} \
			| grep -v "no-packager-tag" \
			| grep -v "invalid-buildhost" \
			| grep -v "no-signature" \
			>> ${BUILD_OUTPUT}/rpmlint/${pkg}
			n_srpms=`rpm -qp --queryformat %{SOURCERPM} ${n} \
				| perl -p -e "s#\.nosrc\.rpm##g" \
				| perl -p -e "s#\.src\.rpm##g"`
			if	[ "${n_srpms}" != "${pkg}" ]
			then
				echo "srpm mismatch `echo ${n} | perl -p -e "s#.*/##g"` ${n_srpms} ${pkg}" \
				>> ${BUILD_OUTPUT}/srpms_mismatch/${pkg}
			fi
		done
		for i in deploop rpmlint
		do
			if [ `cat	${BUILD_OUTPUT}/${i}/${pkg}	| wc -l` = 0 ]
			then
				rm -rf	${BUILD_OUTPUT}/${i}/${pkg}
			fi
		done
		BUILD_RESULT=OK

# "error: cannot open" file does not exist --> remove the buildoutput.
	elif	[ "$(tail -n1 ${BUILD_OUTPUT}/BO/${pkg} \
		| cut -b 0-18)" = "error: cannot open" ];
	then
		rm -rf ${BUILD_OUTPUT}/BO/${pkg}

# "Architecture excluded" --> arch_excl
	elif	[ "$(tail -n2 ${BUILD_OUTPUT}/BO/${pkg} | head -n1 \
		| cut -b 8-19)" = Architecture ];
	then
		BUILD_RESULT=arch_excl

# "source package expected, binary found" --> broken
	elif	[ "$(tail -n2 ${BUILD_OUTPUT}/BO/${pkg} | head -n1 \
		| cut -b 0-37)" = "source package expected, binary found" ];
	then
		BUILD_RESULT=broken

# "cannot be installed" --> cannot_be_installed
# "error: ${RPM_SRC}/${pkg}.${SRC_EXT}.rpm cannot be installed"
# "error: /var/tmp/${pkg}.${SRC_EXT}.rpm cannot be installed"
#	elif	[ "$(head -n2 ${BUILD_OUTPUT}/BO/${pkg} | tail -n1)" \
#		= "error: ${RPM_SRC}/${pkg}.${SRC_EXT}.rpm cannot be installed" ] || \
#		[ "$(head -n2 ${BUILD_OUTPUT}/BO/${pkg} | tail -n1)" \
#		= "error: /var/tmp/${pkg}.${SRC_EXT}.rpm cannot be installed" ]
	elif	[ "$(fgrep -c "error: /var/tmp/${pkg}.${SRC_EXT}.rpm cannot be installed" \
		${BUILD_OUTPUT}/BO/${pkg})" != "0" ]
	then
		BUILD_RESULT=cannot_be_installed

# "Package already exists: %package debug" --> debug
	elif	[ `fgrep -c "error: Package already exists: %package debug" \
		${BUILD_OUTPUT}/BO/${pkg}` != "0" ]
	then
		BUILD_RESULT=debug

# Missing Dependency --> dependency
	elif	[ `grep -c "is needed by" ${BUILD_OUTPUT}/BO/${pkg}` != "0" ]
	then
		BUILD_RESULT=dependency

# "File not found" --> file_not_found
	elif	[ "$(tail -n1 ${BUILD_OUTPUT}/BO/${pkg} \
		| cut -b 5-18)" = "File not found" ];
	then
		BUILD_RESULT=file_not_found
		if	[ "${KEEP_BUILDDIR}" = "1" ]; then
			KEEP_BUILDDIR=2
		fi

# "The following files ought to be marked as %multiarch" --> multiarch
	elif	[ `fgrep -c "The following files ought to be marked as %multiarch" \
		${BUILD_OUTPUT}/BO/${pkg}` != "0" ]
	then
		BUILD_RESULT=multiarch

# "Bad exit status" --> problem
	elif	[ "$(tail -n1 ${BUILD_OUTPUT}/BO/${pkg} \
		| cut -b 5-19)" = "Bad exit status" ];
	then

#	Also see if an "Internal compiler error" took place.
#	If so, make a copy of the Buildoutput
		if	[ `fgrep -c "Internal compiler error" ${BUILD_OUTPUT}/BO/${pkg}` != "0" ]
		then
			cp ${BUILD_OUTPUT}/BO/${pkg} ${HOME}/internal_compiler/${pkg}.`date +%Y-%m-%d_%H:%M:%S`.txt
			echo `date -R` "Internal compiler error" ${pkg} \
			>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
		fi

#	Also see if a "Segmentation fault" took place.
#	If so, make a copy of the Buildoutput
		if	[ `fgrep -c "Segmentation fault" ${BUILD_OUTPUT}/BO/${pkg}` != "0" ]
		then
			cp ${BUILD_OUTPUT}/BO/${pkg} ${HOME}/segfault/${pkg}.`date +%Y-%m-%d_%H:%M:%S`.txt
			echo `date -R` "Segmentation fault" ${pkg} \
			>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
		fi
		if	[ "${KEEP_BUILDDIR}" = "1" ]; then
			KEEP_BUILDDIR=2
		fi
		BUILD_RESULT=problem

# "Installed (but unpackaged) file(s) found:" --> unpackaged_files
	elif	[ `fgrep -c "Installed (but unpackaged) file(s) found:" \
		${BUILD_OUTPUT}/BO/${pkg}` != "0" ]
		then
		BUILD_RESULT=unpackaged_files
		if	[ "${KEEP_BUILDDIR}" = "1" ]; then
			KEEP_BUILDDIR=2
		fi
	fi

if [ "${KEEP_BUILDDIR}" = "2" ]; then
	DATE=`date +%Y-%m-%d_%H:%M:%S`
	pushd ${CHROOT_PATH}${HOME}/RPM/BUILD
	tar cf ~/save/${pkg}.${DATE}.tar *
	bzip2 ~/save/${pkg}.${DATE}.tar
	popd
fi

unset KEEP_BUILDDIR

# Move resulting binary packages.
	if	[ "${KEEP_RPMS}" = "1" ]
	then
		mv -f	~/RPM/RPMS/${P_ARCH}/* \
			~/RPM/RPMS/noarch/* \
			${BUILD_OUTPUT}/RPMS/ 2> /dev/null
	fi

# Remove BuildRequires from chroot
	echo "*** Removing BuildRequires for:" ${pkg} "***"

	R=`diff ~/tmp/begin.txt ${BUILD_OUTPUT}/rpm_qa/${pkg} \
	| grep "^>" \
	| cut -d " " -f2 \
	| perl -p -e "s#\n# #g"`

	if [ "${R}" != "" ]
	then
		echo "*** Removing BuildRequires:" ${R} \
		>> ${BUILD_OUTPUT}/BO/${pkg}

		${SUDO_BIN} ${RPM_BIN} -e -v ${R} \
		--allmatches \
		--root ${CHROOT_PATH} \
		>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1
	fi

	end=`rpm -qa --root ${CHROOT_PATH} | wc -l`

	if [ "$begin" != "${end}" ]
	then
		echo `date -R` begin: ${begin} != end: ${end} \
		>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt

		rpm -qa --root ${CHROOT_PATH} | sort > ~/tmp/${pkg}.txt
		R=`diff ~/tmp/begin.txt ~/tmp/${pkg}.txt \
		| grep "^>" \
		| cut -d " " -f2 \
		| perl -p -e "s#\n# #g"`

		echo "Removing with --noscripts" ${R} \
		>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1

		${SUDO_BIN} ${RPM_BIN} -e -v ${R} \
		--allmatches \
		--noscripts \
		--root ${CHROOT_PATH} \
		>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1

		end=`rpm -qa --root ${CHROOT_PATH} | wc -l`

		if [ "$begin" != "${end}" ]
		then
			echo `date -R` begin: ${begin} != end: ${end} \
			>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
			rm -rf ${LBD}/run/${LOGNAME}@`hostname -f`
		fi
	fi

# Move the buildoutput to the right directory
	if [ "${BUILD_RESULT}" != "" ]
	then
		mv ${BUILD_OUTPUT}/BO/${pkg} \
			${BUILD_OUTPUT}/BO/${BUILD_RESULT}/
	fi

# remove some files left over after removing packages
# packages may be fixed with rpm 4.4 requires(...) ?
	${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} rm -rf /etc/sgml

else
	rm -rf ${LBD}/${P_ARCH}/searching
fi
cleanup ${src} > /dev/null 2>&1
}

update_media () {
# Check if media needs updating
# Input:
# - media
# Output: none
# Configuration file ~/.lbd
# - CHROOT_PATH
# - INSTALLER
# - MEDIA_PATH
# - UPDATE

media=${1}

. ~/.lbd media_${media}

if [ -e ~/.${media} ]
then
# /mirrors/sparc/media/media_info/MD5SUM
# 60e90556835999173641232e80b08762  hdlist_main.cz

# $ md5sum /var/lib/urpmi/hdlist.cooker_main.cz
# c50dbf7b1a10827a9421b71a98a763ac  /var/lib/urpmi/hdlist.cooker_main.cz

#	md5sum_1=`md5sum /var/lib/urpmi/hdlist.${media}.cz | cut -d " " -f1`
#	md5sum_2=`cat /mirrors/Mandrakelinux/devel/cooker/i586/media/media_info/MD5SUM \
#		| grep " hdlist_main.cz" \
#		| cut -d " " -f1`

	if [ `stat -c %Y ${MEDIA_PATH}` \
		!= `cat ~/.${media}` ]
#	if [ "${REGENERATE}" = 1 ]
#	then
#		CURRENT_MTIME=`stat -c %Y \`dirname ${MEDIA_PATH}\`/${RPMS_PATH}`
#	else
#		CURRENT_MTIME=`stat -c %Y ${MEDIA_PATH}`
#	fi
#	
#	if [ x${CURRENT_MTIME} != x`cat ${HOME}/.${media}` ]
	then
		echo "`date -R` updating media ${media}" \
		>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt

#		NEED_UPDATE=1
		
#		if [ "${REGENERATE}" = 1 ]
#		then
#			pushd `dirname ${MEDIA_PATH}`
#			${GENHDLIST_BIN} "${RPMS_PATH}" 2>/dev/null
#			popd
#
#			stat -c %Y \
#			`dirname ${MEDIA_PATH}`/${RPMS_PATH} \
#			> ~/.${media}
#		else
#			stat -c %Y \
#			${MEDIA_PATH} \
#			> ~/.${media}
#		fi

		if	[ "${INSTALLER}" = "apt" ]
		then
			echo "warning: using experimental apt support"
			${SUDO_BIN} ${APT_BIN} \
				update
			if [ "${UPDATE}" = 1 ]
			then
				. ~/.lbd ${CHROOT_PATH}
				echo "`date -R` upgrading chroot ${CHROOT_PATH}" \
				>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
				${SUDO_BIN} ${APT_BIN} \
				dist-upgrade \
				--assume-yes \
				-o RPM::RootDir=${CHROOT_PATH}
			fi
		elif	[ "${INSTALLER}" = "smart" ]
		then
			${SUDO_BIN} ${SMART_BIN} \
				-o force-channels=rpm-sys,${media} \
				update
			if [ "${UPDATE}" = "1" ]
			then
				. ~/.lbd ${CHROOT_PATH}
				echo "`date -R` upgrading chroot ${CHROOT_PATH}" \
				>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
				${SUDO_BIN} ${SMART_BIN} \
					-o force-channels=rpm-sys,${CHROOT_MEDIA} \
					-o rpm-root=${CHROOT_PATH} \
					upgrade \
					-y
			fi

		elif	[ "${INSTALLER}" = "urpmi" ]
		then
			${SUDO_BIN} ${URPMI_UPDATE_BIN} ${media} --no-md5sum
			if	[ "${UPDATE}" = 1 ]
			then
				. ~/.lbd ${CHROOT_PATH}
				echo "`date -R` upgrading chroot ${CHROOT_PATH}" \
				>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
echo "*** Going for: SUDO_BIN=${SUDO_BIN} URPMI_BIN=${URPMI_BIN} CHROOT_MEDIA=${CHROOT_MEDIA} CHROOT_PATH=${CHROOT_PATH} ***" >> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
				${SUDO_BIN} ${URPMI_BIN} -q \
				--auto-select \
				--auto \
				--keep \
				--media ${CHROOT_MEDIA} \
				--no-uninstall \
				--no-verify-rpm \
				--root ${CHROOT_PATH} \
				--split-length 0 \
				>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
			fi
		elif	[ "${INSTALLER}" = "yum" ]
		then
			${YUM_BIN} \
			-c ${YUM_CONFIG} \
			--install-root=${CHROOT_PATH} \
			update
		fi

		stat -c %Y \
		${MEDIA_PATH} \
		> ~/.${media}

		continue
	fi
else
	echo "1" > ~/.${media}
	continue
fi
}

# Main loop

DONE=0

check_config

if [ -e ~/.lbd ]
then
	. ~/.lbd
else
	break
fi

unset OLD_TARGET
if [ "$BUILD_OLD" != "no" ]
then
	OLD_TARGET=old
fi

while		[ -e ${LBD}/run/${LOGNAME}@`hostname -f` ]
do
	for i in `~/.lbd media`
	do
		update_media ${i}
	done
	for j in new $OLD_TARGET
#	for j in new old
	do
		for i in `~/.lbd src`
		do
			new_job=`find_${j} ${i}`
			if  [ "${new_job}" != "" ]
			then
				build ${new_job} ${i}
				DONE=1
				continue 4
			fi
		done
	done
	break
done

if [ "${DONE}" = "1" ]
then
	echo `date -R` "done" >> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
fi
